from flask import current_app
from flask_wtf import FlaskForm
from wtforms import (BooleanField, SelectField, StringField, SubmitField,
                     ValidationError)
from wtforms.validators import DataRequired, Email, Length, Regexp
from ..models import Role, User


class EditUserForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    username = StringField(
        'Username',
        validators=[DataRequired(),
                    Length(1, 64),
                    Regexp(current_app.config['ALLOWED_USERNAME_REGEX'],
                           message='Usernames must have only letters, numbers,'
                                   ' dots or underscores')]
    )
    confirmed = BooleanField('Confirmed')
    role = SelectField(
        'Role',
        choices = [(role.id, role.name)
                   for role in Role.query.order_by(Role.name).all()],
        coerce=int
    )
    submit = SubmitField('Update Profile')

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = user
        self.email.data = self.email.data or user.email
        self.username.data = self.username.data or user.username
        self.confirmed.data = self.confirmed.data or user.confirmed
        self.role.data = self.role.data or user.role_id

    def validate_email(self, field):
        if (field.data != self.user.email
                and User.query.filter_by(email=field.data).first()):
            raise ValidationError('Email already registered.')

    def validate_username(self, field):
        if (field.data != self.user.username
                and User.query.filter_by(username=field.data).first()):
            raise ValidationError('Username already in use.')
